/* contact.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using GLib;
using Gee;
using PeopleBackend;
using PeopleBackendHelper;

namespace PeopleBackendKeyfile {

	public class Contact : PeopleBackendHelper.AbstractContact {
		private static ContactFieldSpec default_field_spec =
				new ContactFieldSpec(ContactFieldFlags.WRITABLE);
		private static ContactFieldSpec mandatory_field_spec =
				new ContactFieldSpec(ContactFieldFlags.WRITABLE | ContactFieldFlags.MANDATORY);

		public string file_path {
			private get;
			private construct set;
		}

		public weak KeyFile key_file {
			get;
			construct;
		}

		construct {
			flags = ContactFlags.EDITABLE | ContactFlags.DELETABLE;
		}

		public Contact(string? id, string? file_path, KeyFile key_file) {
			this.id = id;
			this.file_path = file_path;
			this.key_file = key_file;
		}

		private void save_keyfile () {
			if ((key_file == null) || (file_path == null)) {
				return;
			}

			ulong unused_length;
			var file = FileStream.open(file_path, "w");
			var data = key_file.to_data (out unused_length);
			file.puts (data);
			file = null;
		}

		public override AsyncOperation<Gee.Set<string> > get_defined_fields() {
			if(id == null)
				return sync_result(new HashSet<string>(str_hash, str_equal));

			var result = new Gee.HashSet<string>(str_hash, str_equal);
			string[] fields;
			try {
				fields = key_file.get_keys(id);
				foreach (string field in fields) {
					result.add(field);
				}
			} catch (KeyFileError e) {
				// FIXME: we may want to translate this to a ContactError exception
				//return sync_error(e);
			}

			return sync_result(result);
		}

		public override AsyncOperation<Gee.Set<string> > get_definable_fields() {
			return sync_result(new Gee.HashSet<string>(str_hash, str_equal));
		}

		public override AsyncOperation<Gee.Set<string> > get_mandatory_fields() {
			var result = new Gee.HashSet<string>(str_hash, str_equal);
			result.add ("identity.name.first");
			result.add ("identity.name.last");
			return sync_result(result);
		}

		public override AsyncOperation<ContactFieldSpec> get_field_spec(string field_name) {
			if (field_name == "identity.name.first" || field_name == "identity.name.last")
				return sync_result(mandatory_field_spec);
			return sync_result(default_field_spec);
		}

		public override AsyncOperation<Gee.ReadOnlyMap<string, string> >
				get_fields(Gee.Set<string> fields) {
			var result = new Gee.HashMap<string, string>(str_hash, str_equal, str_equal);
			foreach(string field_name in fields) {
				if (!key_file.has_key(id, field_name))
					continue;
				try {
					var field_value = key_file.get_string(id, field_name);
					if(field_value != null) {
						result.set(field_name, field_value);
					}
				} catch (KeyFileError e) {
				}
			}
			return sync_result(result);
		}

		public override AsyncOperation<Gee.ReadOnlySet<string> >
				set_fields(Gee.Map<string, string> fields) {
			if (id == null) {
				if (!fields.contains ("identity.name.first") || !fields.contains ("identity.name.last"))
					return sync_error(new ContactError.MISSING_MANDATORY_FIELD("Missing mandatory parameters"));
				id = fields.get("identity.name.first") + "-" + fields.get("identity.name.last");
			}

			foreach (weak string name in fields.get_keys()) {
				key_file.set_string(id, name, fields.get(name));
			}
			save_keyfile ();

			var op = sync_result(fields.get_keys());
			return op;
		}

		public override AsyncOperation<bool> @delete() {
			var op = new AsyncOperation<bool>();
			if (id == null)
				op.completed_with_result (false);
			else {
				try {
					key_file.remove_group(id);
					id = null;
					op.completed_with_result(true);
				} catch(Error e) {
					// FIXME: we may want to translate this to a ContactError exception
					//op.completed_with_error(e);
					op.completed_with_result(false);
				}
			}
			save_keyfile ();

			return op;
		}
	}

}
