/* contact-source.vala
 *
 * Copyright (C) 2008 Ali Sabil <ali.sabil@gmail.com>
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using GLib;
using Sqlite;
using Gee;
using PeopleBackend;

namespace PeopleBackendSqlite {

	public class ContactSource : PeopleBackendHelper.AbstractContactSource,
			PeopleBackend.DumpableContactSource,
			PeopleBackend.QueryableContactSource {

		[Notify]
		public ContactSourceFlags flags {
			get;
			protected set;
			default = ContactSourceFlags.CAN_CREATE_CONTACT;
		}

		public string db_path {
			private get;
			construct;
		}

		private Database db;

		construct {
			DirUtils.create_with_parents (Path.get_dirname (db_path), 0700);
			if (Database.open(db_path, out db) != Sqlite.OK) {
				warning("Database '%s' open failed : %s", db_path, db.errmsg);
				db = null;
			} else {
				db.exec (
					"CREATE TABLE IF NOT EXISTS people(" +
						"id integer PRIMARY KEY AUTOINCREMENT," +
						"first_name string," +
						"middle_name string," +
						"last_name string," +
						"home_phone string," +
						"work_phone string," +
						"mobile_phone string," +
						"home_email string," +
						"work_email string" +
					");");
			}
		}

		public ContactSource(string db_path) {
			this.db_path = db_path;
		}


		public override AsyncOperation<PeopleBackend.Contact> request_contact(string? id=null) {
			if (id == null) {
				var contact = new Contact (null, db);
				bind_contact(contact);
				return sync_result(contact);
			}

			var op = async_operation_init();
			var errcode = db.exec("SELECT id, count(*) FROM people WHERE id=%s".printf(id),
					request_contact_callback, op);

			if (errcode != Sqlite.OK) {
				async_operation_error(op, new ContactSourceError.INTERNAL_ERROR(db.errmsg()));
			}

			if (!op.completed) {
				var contact = new Contact(id, db);
				bind_contact(contact);
				async_operation_emit(op, contact);
			}

			return op;
		}

		[NoArrayLength ()]
		private static int request_contact_callback(void* data, int n_columns,
				string[] values, string[] column_names) {
			var op = (AsyncOperation)data;

			if(values[1].to_int() == 0)
				op.completed_with_error(new ContactSourceError.INVALID_ARGUMENT("No such contact"));

			return 0;
		}

		/* DumpableContactSource interface */
		public PeopleBackend.AsyncSearch<Contact> dump() {
			return execute (null);
		}
		/* end of DumpableContactSource interface */

		/* QueryableContactSource interface */
		public AsyncSearch<Contact> execute(Query? query) {
			var search = new AsyncSearch<Contact>();
			var visitor = new SQLQueryVisitor ();

			search.set_data("self", this);
			db.exec(visitor.translate (query), (Sqlite.Callback) query_callback,
					search);
			search.finish();

			return search;
		}

		[NoArrayLength ()]
		private static int query_callback(AsyncSearch search, int n_columns,
				string[] values, string[] column_names) {
			var self = (ContactSource) search.get_data("self");

			for (int i = 0; i < n_columns; i++)
				if (column_names[i] == "id") {
					var contact = new Contact (values[i], self.db);
					search.add_hit(contact);
				}
			return 0;
		}
		/* end of QueryableContactSource interface */
	}

}
