using GLib;
using Gee;

using PeopleBackend;

class PeopleFriend {
	private ContactSourceFactory factory;
	private ContactSource ab;

	private static Gee.Map<string, string> ns_to_desc;

	PeopleFriend() {
		ns_to_desc = new HashMap<string, string>(str_hash, str_equal, str_equal);
		ns_to_desc.set("identity.name.first", "First name");
		ns_to_desc.set("identity.name.middle", "Middle name");
		ns_to_desc.set("identity.name.last", "Last name");
		ns_to_desc.set("communication.phone.home", "Home phone");
		ns_to_desc.set("communication.phone.work", "Work phone");
		ns_to_desc.set("communication.phone.mobile", "Mobile phone");
		ns_to_desc.set("communication.email.home", "Home email");
		ns_to_desc.set("communication.email.work", "Work email");

		factory = new PeopleBackendSqlite.ContactSourceFactory();

		Map<string, string> parameters =
				new HashMap<string, string> (str_hash, str_equal, str_equal);
		parameters.set("db-path", "/tmp/people-sqlite.db");

		try {
			ab = factory.request_contact_source (parameters);
		} catch (PeopleBackend.ContactSourceFactoryError e) {
			stdout.printf ("failed to open the Address Book");
		}
	}

	~PeopleFriend() {
	}

	/* --list */
	public void list() {
		var search = ((DumpableContactSource) ab).dump ();
		search.hit_found += list_hit_found_callback;
		search.done += list_done_callback;
		search.start();
	}

	private static void list_hit_found_callback(AsyncSearch search, Object hit) {
		var contact = (Contact) hit;

		var fields = new Gee.HashSet<string> (str_hash, str_equal);
		fields.add ("identity.name.first");
		contact.get_fields (fields).add_callback ((AsyncOperationCallback) list_first_name_callback, contact);
	}

	private static void list_done_callback(AsyncSearch search) {
		search.stop();
	}

	private static void list_first_name_callback(AsyncOperation<Gee.Map<string, string> >op, Contact c) {
		stdout.printf ("- [%s]\t%s\n", c.id, op.get_result ().get ("identity.name.first"));
	}
	/* end of --list */

	/* --display */
	public void display(string id) {
		ab.request_contact (id).add_callback((AsyncOperationCallback) display_request_contact_callback);
	}

	private static void display_request_contact_callback(AsyncOperation<Gee.Map<string, string> >op) {
		var contact = (Contact) op.get_result ();
		contact.get_fields (ns_to_desc.get_keys ()).add_callback ((AsyncOperationCallback) display_get_fields_callback, contact);
	}

	private static void display_get_fields_callback(AsyncOperation<Gee.Map<string, string> >op, Contact c) {
		try {
			Gee.Map<string, string> fields = op.get_result ();
			stdout.printf ("%s\t[%s]\n", c.id, fields.get ("identity.name.first"));
			foreach (string field in fields.get_keys ()) {
				stdout.printf ("%s: %s\n", ns_to_desc.get (field), fields.get (field));
			}
		} catch (Error e) {
			warning ("%s", e.message);
		}
	}
	/* end of --display */

	/* --create */
	public void create() {
		ab.request_contact (null).add_callback((AsyncOperationCallback) create_request_contact_callback);
	}

	private static void create_request_contact_callback(AsyncOperation<Gee.Map<string, string> >op) {
		var contact = (Contact) op.get_result ();
		contact.get_definable_fields ().add_callback ((AsyncOperationCallback) create_get_definable_fields_callback, contact);
	}

	private static void create_get_definable_fields_callback(AsyncOperation<Gee.Set<string> > op, Contact contact) {
		try {
			var definable = op.get_result();
			contact.set_data("definable", definable);
			do_create_contact(contact);
		} catch (Error e) {
			warning ("%s", e.message);
		}
	}

	private static void do_create_contact(Contact contact) {
		Gee.Map<string, string> fields = new HashMap<string, string> (str_hash, str_equal, str_equal);
		var definable = (Gee.Set<string>)contact.get_data("definable");

		foreach(string field in definable) {
			var value = get_entry ("%s: ".printf (ns_to_desc.get (field)));

			if(value != null)
				fields.set (field, value);
		}

		contact.set_fields (fields).add_callback ((AsyncOperationCallback) create_contact_callback, contact);
	}

	private static void create_contact_callback(AsyncOperation<Gee.Set<string> > op, Contact c) {
		stdout.printf ("Contact created with %s.\n", c.id);
	}
	/* end of --create */

	/* --delete */
	public void delete_(string id) {
		ab.request_contact (id).add_callback((AsyncOperationCallback) delete_request_contact_callback);
	}

	private static void delete_request_contact_callback(AsyncOperation<Gee.Map<string, string> >op) {
		var contact = (Contact) op.get_result ();
		contact.delete ().add_callback ((AsyncOperationCallback) delete_contact_callback, contact.id);
	}

	private static void delete_contact_callback(AsyncOperation<bool> op, string id) {
		stdout.printf ("Contact %s deleted.\n", id);
	}
	/* end of --delete */

	/* --modify */
	public void modify(string id) {
		ab.request_contact (id).add_callback((AsyncOperationCallback) modify_request_contact_callback);
	}

	private static void modify_request_contact_callback(AsyncOperation<Gee.Map<string, string> >op) {
		var contact = (Contact) op.get_result ();
		contact.get_definable_fields ().add_callback ((AsyncOperationCallback) modify_get_definable_fields_callback, contact);
	}

	private static void modify_get_definable_fields_callback(AsyncOperation<Gee.Set<string> > op, Contact contact) {
		contact.set_data("definable", op.get_result());
		contact.get_defined_fields ().add_callback ((AsyncOperationCallback) modify_get_defined_fields_callback, contact);
	}

	private static void modify_get_defined_fields_callback(AsyncOperation<Gee.Set<string> > op, Contact c) {
		c.get_fields (op.get_result ()).add_callback ((AsyncOperationCallback) modify_get_fields_callback, c);
	}

	private static void modify_get_fields_callback(AsyncOperation<Gee.Map<string, string> > op, Contact c) {
		do_modify_contact (c, op.get_result ());
	}

	private static void do_modify_contact(Contact contact, Gee.Map<string, string> defined_fields) {
		Gee.Map<string, string> fields = new HashMap<string, string> (str_hash, str_equal, str_equal);
		var definable = (Gee.Set<string>)contact.get_data("definable");

		foreach(string field in definable) {
			string current = defined_fields.get (field);
			if(current == null) current = "undefined";

			var value = get_entry ("%s [%s]: ".printf (ns_to_desc.get (field), current));

			if(value != null) {
				if(value == "*") value = null;
				fields.set (field, value);
			}
		}

		contact.set_fields (fields).add_callback ((AsyncOperationCallback) modify_contact_callback, contact.id);
	}

	private static void modify_contact_callback(AsyncOperation<Gee.Map<string, string> > op, string id) {
		stdout.printf ("Contact %s was modified.\n", id);
	}
	/* end of --modify */

	/**
	 * Gets a text entry from the user.
	 *
	 * @param prompt the message prompted to the user
	 * @return       a string containing the user entry or <code>null</code> if
	 *               no entry was specified by the user
	 */
	private static string? get_entry(string? prompt=null) {
		var s = new GLib.StringBuilder ();

		if (prompt != null)
			stdout.printf (prompt);

		while (true) {
			var c = (char) stdin.getc();
			if (c == '\n')
				break;
			s.append_c(c);
		}

		if (s.len == 0)
			return null;
		return s.str;
	}

	public static int main (string[] args) {
		PeopleFriend friend = new PeopleFriend();

		foreach (string arg in args) {
			if (arg == "--list") {
				friend.list ();
				return 0;
			} else if (arg == "--create") {
				friend.create ();
				return 0;
			} else if (arg == "--delete") {
				var id = get_entry ("Contact id: ");

				if (id != null)
					friend.delete_ (id);
				return 0;
			} else if (arg == "--modify") {
				var id = get_entry ("Contact id: ");

				if (id != null)
					friend.modify (id);
				return 0;
			} else if (arg == "--display") {
				var id = get_entry ("Contact id: ");

				if (id != null)
					friend.display (id);
				return 0;

			}
		}

		stdout.printf("Available options are:\n\n");
		stdout.printf("\t--list        lists the contacts\n");
		stdout.printf("\t--display     displays informations for a contact\n");

		stdout.printf("\v\t--create      creates a new contact\n");
		stdout.printf("\t--delete      deletes a contact\n");
		stdout.printf("\t--modify      modifies a contact (enter * to delete a field)\n");

		stdout.printf("\v\t--help        displays that help\n\n");

		return 0;
	}

}
