/* service.vala
 *
 * Copyright (C) 2008 Ali Sabil <ali.sabil@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using GLib;
using Gee;
using People;


namespace People {

	[DBus (name = "net.launchpad.People")]
	public class Service : PeopleService.DBusObject {

		[DBus (hidden = true)]
		public string config_file {
			get;
			private set construct;
		}

		private AddressBookManager address_book_manager;
		private Map<string, PeopleService.AddressBook> address_books;

		construct {
			var config = new ConfigXml (config_file);
			address_book_manager = new AddressBookManager (config);
			address_books = new HashMap<string, PeopleService.AddressBook> (str_hash, str_equal);
		}

		public Service (DBus.Connection connection, string config_file) {
			this.connection = connection;
			this.object_path = "/net/launchpad/People";
			this.config_file = config_file;
		}

		[NoArrayLength]
		public void ListAddressBooks(out string[] address_books) throws Error {
			var address_books_set = address_book_manager.list_address_books ();
			address_books = new string[address_books_set.size];
			uint i = 0;

			foreach (string address_book in address_books_set) {
				address_books[i] = address_book;
				i++;
			}
		}

		[DBus (object_path_signature="o")]
		public void RequestAddressBook(string address_book_id,
				out string service_name, out string object_path) throws Error {
			PeopleService.AddressBook ab;
			if (address_books.contains (address_book_id)) {
				ab = address_books.get (address_book_id);
			} else {
				var people_ab = address_book_manager.get_address_book (address_book_id);
				if (people_ab == null) {
					// FIXME: we want to throw an error
					warning ("Requested unknown Address Book");
				}
				ab = new PeopleService.AddressBook (connection,
						address_book_id, people_ab);
				address_books.set (address_book_id, ab);
			}
			service_name = ab.bus_name;
			object_path = ab.object_path;
		}
	}
}


private static class ConfigManager {

	public static string get_config_path () {
		return Path.build_filename (Environment.get_user_config_dir (),
		                            "people",
		                            "address-books.xml");
	}

	public static void init () {
		string config_path = get_config_path ();
		if (FileUtils.test (config_path, FileTest.EXISTS)) {
			return;
		}

		DirUtils.create_with_parents (Path.get_dirname (config_path), 0700);
		var config_file = FileStream.open(config_path, "w+");
		config_file.puts ("<people>\n");
		config_file.puts ("    <contact-sources>\n");
		config_file.puts ("        <contact-source id=\"local-db\" class=\"sqlite\">\n");
		config_file.puts ("            <param name=\"db-path\">sqlite/main.db</param>\n");
		config_file.puts ("        </contact-source>\n");
		config_file.puts ("\n");
		config_file.puts ("        <contact-source id=\"local-keyfile\" class=\"keyfile\">\n");
		config_file.puts ("            <param name=\"file-path\">keyfile/main.ini</param>\n");
		config_file.puts ("        </contact-source>\n");
		config_file.puts ("    </contact-sources>\n");
		config_file.puts ("\n");
		config_file.puts ("    <address-books>\n");
		config_file.puts ("        <address-book id=\"main\">\n");
		config_file.puts ("            <contact-source id=\"local-db\"/>\n");
		config_file.puts ("            <contact-source id=\"local-keyfile\"/>\n");
		config_file.puts ("        </address-book>\n");
		config_file.puts ("    </address-books>\n");
		config_file.puts ("</people>\n");
		config_file.flush ();
		config_file = null;

		// Inject some data into the keyfile
		string keyfile_path = Path.build_filename (Environment.get_user_data_dir (), "people", "keyfile/main.ini");
		if (FileUtils.test (keyfile_path, FileTest.EXISTS)) {
			return;
		}

		DirUtils.create_with_parents (Path.get_dirname (keyfile_path), 0700);

		var keyfile_file = FileStream.open(keyfile_path, "w+");
		keyfile_file.puts ("[people.hackers]\n");
		keyfile_file.puts ("identity.name.first = People\n");
		keyfile_file.puts ("identity.name.last = Hackers\n");
		keyfile_file.puts ("communication.email.work = people.hackers@rocks.org\n");
		keyfile_file.puts ("\n");
		keyfile_file.puts ("[people.users]\n");
		keyfile_file.puts ("identity.name.first = People\n");
		keyfile_file.puts ("identity.name.last = Users\n");
		keyfile_file.puts ("communication.email.work = people.users@rocks.org\n");
		keyfile_file.flush ();
		keyfile_file = null;


	}
}


int main () {
	var loop = new MainLoop (null, false);
	var conn = DBus.Bus.get (DBus.BusType.SESSION);

	dynamic DBus.Object bus = conn.get_object ("org.freedesktop.DBus",
	                                           "/org/freedesktop/DBus",
	                                           "org.freedesktop.DBus");

	// try to register service in session bus
	uint result = bus.RequestName ("net.launchpad.People", (uint) 0);
	if (result == DBus.RequestNameReply.PRIMARY_OWNER) {
		ConfigManager.init ();
		var service = new People.Service (conn, ConfigManager.get_config_path ());
		loop.run ();
	} else {
		warning ("People service already running. Exiting.");
		return 1;
	}
	return 0;
}
