/* async-operation.vala
 *
 * Copyright (C) 2008 Ali Sabil <ali.sabil@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using GLib;


namespace PeopleUtils {

	public errordomain AsyncOperationError {
		RESULT_NOT_AVAILABLE = 1,
	}

	private class AsyncOperationContext : Object {
		public AsyncOperationCallback callback;
		public void* user_data;
	}

	/**
	 * Class representing an asynchronous operation, an instance of this class
	 * is returned by asynchronous methods.
	 *
	 * @param <T> a type for the result of the AsyncOperation
	 */
	public class AsyncOperation<T> : Object {

		/**
		 * AsyncOperation completion state.
		 * <p>
		 * Its value is <code>true</code> if the asynchronous operation is
		 * completed yet, <code>false</code> otherwise.
		 */
		[Notify]
		public bool completed {
			get;
			private set;
			default =false;
		}

		/**
		 * AsyncOperation success state.
		 * <p>
		 * Its value is <code>true</code> if the asynchronous operation
		 * succeeded, <code>false</code> if an error occured.
		 */
		public bool succeeded {
			get {
				return (completed && error == null);
			}
		}

		private T result = null;
		private Error error = null;
		private string error_message = null;

		private Gee.List<AsyncOperationContext> callbacks = new Gee.ArrayList<AsyncOperationContext> ();

		/**
		 * Returns the result of this AsyncOperation.
		 *
		 * @return the result of the asynchronous operation
		 * @throws {@link AsyncOperationError#RESULT_NOT_AVAILABLE} if the
		 *         result of the asynchronous operation isn't available yet,
		 *         {@link AsyncOperationError#OPERATION_FAILED} if the
		 *         processing of the asynchronous operation led to a failure
		 */
		public T get_result() throws Error {
			if (!completed)
				throw new AsyncOperationError.RESULT_NOT_AVAILABLE ("Async operation not yet completed");
			if (error != null)
				throw error;
			return this.result;
		}

		/**
		 * Adds an {@link AsyncOperationCallback} to be called by this
		 * AsyncOperation when it gets completed.
		 * <p>
		 * If this AsyncOperation was already completed, the given
		 * AsyncOperationCallback will be immediatly called.
		 *
		 * @param callback  AsyncOperationCallback to be called
		 * @param user_data user provided data to be passed to the
		 *                  AsyncOperationCallback when called
		 * @see             AsyncOperationCallback
		 */
		public AsyncOperation<T> add_callback(AsyncOperationCallback callback, void* user_data=null) {
			if (completed)
				callback (this, user_data);
			else {
				var callback_struct = new AsyncOperationContext ();
				callback_struct.callback = callback;
				callback_struct.user_data = user_data;
				callbacks.add (callback_struct);
			}
			return this;
		}

		/**
		 * Completes this AsyncOperation with the given result and runs the
		 * attached {@link AsyncOperationCallback}.
		 * <p>
		 * That method won't have any effect if this AsyncOperation was already
		 * completed.
		 *
		 * @param res the result to complete this AsynOperation with
		 */
		public void completed_with_result(T res) {
			return_if_fail (_completed == false);

			result = res;
			completed = true;
			run_callbacks ();
		}

		/**
		 * Completes this AsyncOperation with the given error message and runs
		 * the attached {@link AsyncOperationCallback}.
		 * <p>
		 * That method won't have any effect if this AsyncOperation was already
		 * completed.
		 *
		 * @param error_message the error message to complete this
		 *                      AsynOperation with
		 */
		public void completed_with_error(Error err) {
			return_if_fail (completed == false);

			error = err;
			completed = true;
			run_callbacks ();
		}

		private void run_callbacks() {
			foreach (weak AsyncOperationContext cb in callbacks)
				cb.callback (this, cb.user_data);
			callbacks.clear ();
		}
	}

	/**
	 * Delegate method defining {@link AsyncOperation} callbacks signature.
	 *
	 * @param op        the AsyncOperation to be passed to the callback
	 * @param data      user provided data to be passed to the callback
	 * @see             AsyncOperation#add_callback
	 */
	public delegate void AsyncOperationCallback(AsyncOperation op, void* data);

}
