/* async-search.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 * Copyright (C) 2008 Ali Sabil <ali.sabil@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using GLib;
using Gee;


namespace PeopleUtils {

	/**
	 * Class representing an asynchronous search, an instance of this class
	 * is returned by asynchronous query methods.
	 */
	public class AsyncSearch<T> : Object {
		private bool started = false;
		private bool finished = false;

		private Gee.List<T> buffered_hits;

		/**
		 * Signal emitted when a new hit is found.
		 *
		 * @param hit the new hit found
		 */
		public signal void hit_found(T hit);

		/**
		 * Signal emitted when most of the hits for this AsyncSearch have been
		 * found.
		 */
		public signal void done();

		/**
		 * Makes this AsyncSearch start emitting signals.
		 * <p>
		 * Buffered hits found before this method is called are now propagated
		 * through several {@link AsyncSearch#hit_found} emissions.
		 * <p>
		 * If the AsyncSearch was marked as done before this method is called,
		 * then {@link AsyncSearch#done} is emitted.
		 */
		public void start() {
			return_if_fail (started != true);
			started = true;

			if (buffered_hits != null) {
				foreach(weak T hit in buffered_hits)
					hit_found(hit);
				buffered_hits.clear();
			}

			if (finished)
				done();
		}

		/**
		 * Makes this AsyncSearch stop emitting signals.
		 *
		 * FIXME: for now, only suspends the signal emissions, as the hits
		 *        are buffered again. If this is the wanted behavior, we
		 *        should change the name of that method. If the wanted
		 *        behavior is to actually stop the research for new hits,
		 *        then we need to find a way to do so.
		 */
		public void stop() {
			started = false;
		}

		/**
		 * Marks this AsyncSearch as finished and emits
		 * {@link AsyncSearch#done}.
		 * <p>
		 * This method should only be used from the entity populating this
		 * AsyncSearch.
		 */
		public void finish() {
			finished = true;
			if (started)
				done();
		}

		/**
		 * Adds a hit to this AsyncSearch.
		 * <p>
		 * If this AsyncSearch is started, then the hit is propagated through
		 * a {@link AsyncSearch#hit_found} emission. Else, the hit is buffered.
		 * <p>
		 * This method should only be used from the entity populating this
		 * AsyncSearch.
		 */
		public void add_hit(T hit) {
			if (started)
				hit_found(hit);
			else {
				if (buffered_hits == null)
					buffered_hits = new Gee.ArrayList<T>();
				buffered_hits.add(hit);
			}
		}
	}

}
