/* multi-async-operation.vala
 *
 * Copyright (C) 2008 Ali Sabil <ali.sabil@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using GLib;
using Gee;


namespace PeopleUtils {

	private class MultiAsyncOperationContext : Object {
		public MultiAsyncOperationCallback callback;
		public void* user_data;
	}

	public class MultiAsyncOperation<T> : Object {

		public Gee.List<AsyncOperation<T> > operations {
			private get;
			private set construct;
		}

		private Gee.List<MultiAsyncOperationContext> callbacks =
				new Gee.ArrayList<MultiAsyncOperationContext> ();

		private T[] results;
		private bool[] valid_results;
		private int completed = 0;

		construct {
			assert (operations.size > 0);

			results = new T[operations.size];
			valid_results = new bool[operations.size];

			completed = 0;

			// add_callback on each operation
			foreach (AsyncOperation<T> operation in operations) {
				operation.add_callback (operation_callback, this);
			}
		}

		public MultiAsyncOperation(Gee.List<AsyncOperation<T> > operations) {
			this.operations = operations;
		}

		public MultiAsyncOperation<T> add_callback(MultiAsyncOperationCallback callback, void* user_data=null) {
			if (completed == operations.size)
				callback (this, user_data);
			else {
				var callback_struct = new MultiAsyncOperationContext ();
				callback_struct.callback = callback;
				callback_struct.user_data = user_data;
				callbacks.add (callback_struct);
			}
			return this;
		}

		public Gee.List<T?> get_result() throws Error {
			if (completed != operations.size)
				throw new AsyncOperationError.RESULT_NOT_AVAILABLE ("Async operation not yet completed");

			var result = new Gee.ArrayList<T?> ();

			for (int i = 0; i < operations.size; i++) {
				if (valid_results[i]) {
					result.insert (i, results[i]);
				} else {
					result.insert (i, null);
				}
			}

			return result;
		}

		private static void operation_callback(AsyncOperation<T> operation, void* data) {
			MultiAsyncOperation<T> self = (MultiAsyncOperation<T>) data;
			int operation_index = self.operations.index_of (operation);

			self.completed++;
			if (operation.succeeded) { // Vala Exceptions don't seem to be reliable
				self.valid_results[operation_index] = true;
				self.results[operation_index] = (string) operation.get_result();
			}

			if (self.completed == self.operations.size) {
				self.run_callbacks();
			}
		}

		private void run_callbacks() {
			foreach (weak MultiAsyncOperationContext cb in callbacks)
				cb.callback (this, cb.user_data);
			callbacks.clear ();
		}

	}

	public delegate void MultiAsyncOperationCallback(MultiAsyncOperation op, void* data);

}
