/* async-operation-helper.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using GLib;
using Gee;
using PeopleBackend;


namespace PeopleBackendHelper {
	/**
	 * This class provides useful methods to deal with {@link AsyncOperation}
	 * creation and completion. To be extended by classes managing
	 * AsyncOperation results.
	 */
	public class AsyncOperationHelper : Object  {
		/**
		 * {@link AsyncOperation} instances created from this
		 * AsyncOperationHelper are kept in reference in that {@link Set}
		 * during their validity time.
		 */
		protected Set<AsyncOperation> pending_operations = new HashSet<AsyncOperation>();

		/**
		 * Returns an {@link AsyncOperation} completed with the given Object
		 * result in a synchronous way.
		 *
		 * @param object Object to immediatly complete the AsyncOperation with
		 * @return       an AsyncOperation completed with the given Object
		 */
		protected AsyncOperation<Object> sync_result(Object object) {
			var op = new AsyncOperation<Object>();
			op.completed_with_result(object);
			return op;
		}

		/**
		 * Returns an {@link AsyncOperation} completed with the given boolean
		 * result in a synchronous way.
		 *
		 * @param boolean bool to immediatly complete the AsyncOperation with
		 * @return        an AsyncOperation completed with the given bool
		 */
		protected AsyncOperation<bool> sync_result_bool(bool boolean) {
			var op = new AsyncOperation<bool>();
			op.completed_with_result(boolean);
			return op;
		}

		/**
		 * Returns an {@link AsyncOperation} completed with the given error
		 * in a synchronous way.
		 *
		 * @param error_message error message to immediatly complete the
		 *                      AsyncOperation with
		 * @return              an AsyncOperation completed with the given
		 *                      error message
		 */
		protected AsyncOperation<void*> sync_error(Error error) {
			var op = new AsyncOperation<void*>();
			op.completed_with_error(error);
			return op;
		}

		/**
		 * Instanciates an {@link AsyncOperation} to be completed with an
		 * Object result in an asynchronous way.
		 *
		 * @return the pending AsyncOperation
		 */
		protected AsyncOperation<Object> async_operation_init() {
			var op = new AsyncOperation<Object>();
			pending_operations.add(op);
			return op;
		}

		/**
		 * Instanciates an {@link AsyncOperation} to be completed with a
		 * boolean result in an asynchronous way.
		 *
		 * @return the pending AsyncOperation
		 */
		protected AsyncOperation<bool> async_operation_init_bool() {
			var op = new AsyncOperation<bool>();
			pending_operations.add(op);
			return op;
		}

		/**
		 * Completes the given pending {@link AsyncOperation} with the given
		 * Object result.
		 *
		 * @param op     the pending AsyncOperation to be completed
		 * @param object the Object to complete the AsyncOperation
		 */
		protected void async_operation_emit(AsyncOperation<Object> op,
				Object object) {
			pending_operations.remove(op);
			op.completed_with_result(object);
		}

		/**
		 * Completes the given pending {@link AsyncOperation} with the given
		 * boolean result.
		 *
		 * @param op      the pending AsyncOperation to be completed
		 * @param boolean the bool used to complete the AsyncOperation
		 */
		protected void async_operation_emit_bool(AsyncOperation<Object> op,
				bool boolean) {
			pending_operations.remove(op);
			op.completed_with_result(boolean);
		}

		/**
		 * Completes the given pending {@link AsyncOperation} with the given
		 * error.
		 *
		 * @param op            the pending AsyncOperation to be completed
		 * @param error_message the error message to use to complete the
		 *                      AsyncOperation
		 */
		protected void async_operation_error(AsyncOperation<void*> op,
				Error error) {
			pending_operations.remove(op);
			op.completed_with_error(error);
		}
	}
}
