/* contact-source.vala
 *
 * Copyright (C) 2008 Ali Sabil <ali.sabil@gmail.com>
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 * Copyright (C) 2008 Quentin Bonnard <quentin.bonnard@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using GLib;

namespace PeopleBackend {

	public errordomain ContactSourceError {
		NOT_AVAILABLE = 1,
		INVALID_ARGUMENT = 2,

		INTERNAL_ERROR = 99,
	}

	[Flags]
	public enum ContactSourceFlags {
		NONE               = 0,
		CAN_CREATE_CONTACT = 1 << 0,
		NEED_NETWORK       = 1 << 1
	}

	/**
	 * This interface defines base methods and properties to be implemented by
	 * all the contact source backends.
	 */
	public interface ContactSource : Object {
		/**
		 * A combination of {@link ContactSourceFlags}.
		 */
		[Notify]
		public abstract ContactSourceFlags flags {
			get;
			protected set;
			default = ContactSourceFlags.NONE;
		}

		/**
		 * Signal emitted when a new {@link Contact} is added to this
		 * ContactSource.
		 *
		 * @param contact the corresponding Contact
		 */
		public signal void contact_added(Contact contact);

		/**
		 * Signal emitted when a new {@link Contact} is removed from this
		 * ContactSource.
		 *
		 * @param contact the corresponding Contact
		 */
		public signal void contact_removed(Contact contact);

		/**
		 * Returns the {@link Contact} corresponding to the given id.
		 * <p>
		 * If the given id is <code>null</code>, and the backend allows contact
		 * creation, a new empty contact is created.
		 *
		 * @param id the {@link Contact#id} to request an existing Contact, or
		 *           <code>null</code> to request a new Contact
		 * @return   an {@link AsyncOperation} to be completed with a Contact
		 * @throws   {@see ContactSourceError#INVALID_ARGUMENT} is the provided
		 *           id does not match any existing Contact or if the creation
		 *           of a new Contact is not allowed
		 * @see      Contact
		 * @see      Contact#id
		 */
		public abstract AsyncOperation<Contact> request_contact(string? id);
	}

}
