/* bimap.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using GLib;
using Gee;


namespace People {
	/**
	 * A BiMap defines a bidirectionnal mapping relationship between elements of
	 * classes {@link X} and {@link Y}.
	 */
	public class BiMap<X, Y> : Object {

		public GLib.HashFunc x_hash_func {
			get;
			private set construct;
		}

		public GLib.EqualFunc x_equal_func {
			get;
			private set construct;
		}

		public GLib.HashFunc y_hash_func {
			get;
			private set construct;
		}

		public GLib.EqualFunc y_equal_func {
			get;
			private set construct;
		}

		public Gee.Map<X, Y> x_to_y {
			get;
			private set construct;
		}

		public Gee.Map<X, Y> y_to_x {
			get;
			private set;
		}

		construct {
			if (x_to_y == null)
				x_to_y = new Gee.HashMap<X, Y> (x_hash_func, x_equal_func, y_equal_func);
		}

		/**
		 * Constructs this BiMap based on the given {@link Map}.
		 *
		 * @param x_to_y       a mapping of {@link X} elements to {@link Y}
		 *                     elements
		 * @param x_hash_func  hash function for elements of {@link X}
		 * @param x_equal_func equal function for elements of {@link X}
		 * @param y_hash_func  hash function for elements of {@link Y}
		 * @param y_equal_func equal function for elements of {@link Y}
		 */
		public BiMap (Gee.Map<X, Y>? x_to_y=null,
				GLib.HashFunc x_hash_func = GLib.direct_hash,
				GLib.EqualFunc x_equal_func = GLib.direct_equal,
				GLib.HashFunc y_hash_func = GLib.direct_hash,
				GLib.EqualFunc y_equal_func= GLib.direct_equal) {
			this.x_to_y = x_to_y;
			this.x_hash_func = x_hash_func;
			this.x_equal_func = x_equal_func;
			this.y_hash_func = y_hash_func;
			this.y_equal_func = y_equal_func;
		}

		/**
		 * Returns the image of the given element under this BiMap.
		 *
		 * @param x an element of {@link X}
		 * @return  an element of {@link Y} or <code>NULL</code> of the
		 *          relationship does not exist
		 */
		public Y get_y (X x) {
			return x_to_y.get (x);
		}

		/**
		 * Returns the image of the given element under this BiMap.
		 *
		 * @param x an element of {@link Y}
		 * @return  an element of {@link X} or <code>NULL</code> of the
		 *          relationship does not exist
		 */
		public X get_x (Y y) {
			if (y_to_x == null) {
				y_to_x = new Gee.HashMap<Y, X> (y_hash_func, y_equal_func, x_equal_func);

				foreach (X x in x_to_y.get_keys ()) {
					y_to_x.set ((Y) (x_to_y.get (x)), x);
				}
			}
			return y_to_x.get (y);
		}

		/**
		 * Establishes a bidirectionnal mapping relationship between the given
		 * elements under this BiMap.
		 *
		 * @param x an element of {@link X}
		 * @param y an element of {@link Y}
		 */
		public void set_x (X x, Y y) {
			if (y_to_x == null) {
				y_to_x = new Gee.HashMap<Y, X> (y_hash_func, y_equal_func, x_equal_func);

				foreach (X x in x_to_y.get_keys ()) {
					y_to_x.set ((Y) (x_to_y.get (x)), x);
				}
			}
			if (x_to_y.contains(x)) {
				y_to_x.remove (x_to_y.get(x));
				x_to_y.remove (x);
			}
			if (y_to_x.contains(y)) {
				x_to_y.remove (y_to_x.get(y));
				y_to_x.remove (y);
			}
			x_to_y.set (x, y);
			y_to_x.set (y, x);
		}

		/**
		 * Establishes a bidirectionnal mapping relationship between the given
		 * elements under this BiMap.
		 *
		 * @param y an element of {@link Y}
		 * @param x an element of {@link X}
		 */
		public void set_y (Y y, X x) {
			set_x (x, y);
		}

		/**
		 * Removes the relationship involving the given element of {@link X}.
		 *
		 * @param x an element of {@link X}
		 */
		public void remove_x (X x) {
			if (y_to_x != null)
				y_to_x.remove (x_to_y.get (x));
			x_to_y.remove (x);
		}

		/**
		 * Removes the relationship involving the given element of {@link Y}.
		 *
		 * @param y an element of {@link Y}
		 */
		public void remove_y (Y y) {
			if (y_to_x == null) {
				y_to_x = new Gee.HashMap<Y, X> (y_hash_func, y_equal_func, x_equal_func);

				foreach (X x in x_to_y.get_keys ()) {
					y_to_x.set ((Y) (x_to_y.get (x)), x);
				}
			}
			x_to_y.remove (y_to_x.get (y));
			y_to_x.remove (y);
		}

		/**
		 * Returns the sub-set of {@link X} of elements used in this BiMap.
		 *
		 * @return a {@link Set} of elements of {@link X}
		 */
		public Gee.Set<X> get_x_set () {
			return new Gee.ReadOnlySet<X> (x_to_y.get_keys ());
		}

		/**
		 * Returns the sub-set of {@link Y} of elements used in this BiMap.
		 *
		 * @return a {@link Set} of elements of {@link Y}
		 */
		public Gee.Set<Y> get_y_set () {
			if (y_to_x == null) {
				y_to_x = new Gee.HashMap<Y, X> (y_hash_func, y_equal_func, x_equal_func);

				foreach (X x in x_to_y.get_keys ()) {
					var y = x_to_y.get (x);
					y_to_x.set (y, x);
				}
			}
			return new Gee.ReadOnlySet<Y> (y_to_x.get_keys ());
		}
	}

}
