/* query-builder.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using GLib;
using Gee;
using PeopleBackend;


namespace People {
	/**
	 * QueryBuilder objects are used to generate a {@link Query} tree from a
	 * string-formatted query.
	 */
	public class QueryBuilder : Object {
		private enum Symbol {
			NOT = 271, // TokenType.LAST + 1
			AND,
			OR,
			EQUALS,
			CONTAINS
		}

		private Scanner scanner;

		construct {
			var config = ScannerConfig ();
			config.cset_skip_characters = " \t\r\n";
			config.cset_identifier_first = "abcdefghijklmnopqrstuvwxyz" +
					"ABCDEFGHIJKLMNOPQRSTUVWXYZ";
			config.cset_identifier_nth = "abcdefghijklmnopqrstuvwxyz" + "." +
					"ABCDEFGHIJKLMNOPQRSTUVWXYZ" + "0123456789";
			config.cpair_comment_single = ";\n";
			config.case_sensitive = 0;

			config.skip_comment_multi = 1;
			config.skip_comment_single = 1;
			config.scan_comment_multi = 1;

			config.scan_identifier = 1;
			config.scan_identifier_1char = 1;
			config.scan_identifier_NULL = 1;
			config.scan_symbols = 1;

			config.scan_string_sq = 1;
			config.scan_string_dq = 1;
			config.numbers_2_int = 1;
			config.int_2_float = 0;
			config.identifier_2_string = 0;
			config.char_2_token = 1;
			config.symbol_2_token = 0;
			config.scope_0_fallback = 0;

			scanner = new Scanner (config);
			var scope = 0;
			scanner.scope_add_symbol (scope, "NOT", (void *) Symbol.NOT);
			scanner.scope_add_symbol (scope, "AND", (void *) Symbol.AND);
			scanner.scope_add_symbol (scope, "OR", (void *) Symbol.OR);
			scanner.scope_add_symbol (scope, "EQUALS", (void *) Symbol.EQUALS);
			scanner.scope_add_symbol (scope, "CONTAINS", (void *) Symbol.CONTAINS);
			scanner.set_scope (scope);
		}

		/**
		 * Builds the {@link Query} tree corresponding to the given query.
		 *
		 * @param query a S-Expression formatted query
		 * @return      a Query tree
		 */
		public Query build (string query) {
			Query structure = null;
			scanner.input_text (query, (uint) query.size);
			structure = parse_expression ();
//			if (!scanner.eof ()) {
//				// FIXME: throw an exception
//				structure = null;
//			}
			return structure;
		}

		private Query? parse_expression () {
			Query query = null;
			var token = scanner.get_next_token ();

			switch (token) {
			case TokenType.EOF:
				break;
			case TokenType.LEFT_PAREN:
				query = parse_list ();
				break;
			case TokenType.IDENTIFIER:
				var str_ident = scanner.cur_value ().v_identifier;
				query = new FieldQuery (str_ident);
				break;
			case TokenType.STRING:
				var str_value = scanner.cur_value ().v_string;
				query = new ValueQuery (str_value);
				break;
			case TokenType.INT:
				var str_value = scanner.cur_value ().v_int.to_string ();
				query = new ValueQuery (str_value);
				break;
			default:
				// FIXME: throw an exception
				return null;
			}

			return query;
		}

		private Query? parse_list () {
			Query query = null;
			var token = scanner.get_next_token ();

			switch (token) {
			case TokenType.SYMBOL:
				var symbol = (Symbol) scanner.cur_value ().v_symbol;
				var expressions = parse_expressions ();
				token = scanner.get_next_token (); /* consumes the left ')' */
				if (token != TokenType.RIGHT_PAREN) {
					// FIXME: throw an exception
					return null;
				}

				switch (symbol) {
				// TODO: check if the length of the list is OK with the symbol
				case Symbol.NOT:
					if (expressions.size != 1) {
						// FIXME: throw an exception
						return null;
					}
					query = new CompositeQuery (Operator.NOT, expressions);
					break;
				case Symbol.AND:
					if (expressions.size <= 1) {
						// FIXME: throw an exception
						return null;
					}
					query = Query.and (expressions);
					break;
				case Symbol.OR:
					if (expressions.size <= 1) {
						// FIXME: throw an exception
						return null;
					}
					query = Query.or (expressions);
					break;
				case Symbol.EQUALS:
					if (expressions.size != 2) {
						// FIXME: throw an exception
						return null;
					}
					query = new CompositeQuery (Operator.EQUALS, expressions);
					break;
				case Symbol.CONTAINS:
					if (expressions.size != 2) {
						// FIXME: throw an exception
						return null;
					}
					query = new CompositeQuery (Operator.CONTAINS, expressions);
					break;
				default:
					break;
				}
				break;
			default:
				// FIXME: throw an exception
				return null;
				break;
			}

			return query;
		}

		private Gee.List<Query> parse_expressions () {
			var expressions = new Gee.ArrayList<Query> ();
			TokenType token;

			while (((token = scanner.peek_next_token ()) != TokenType.RIGHT_PAREN) &&
					(token != TokenType.EOF)) {
				var expression = parse_expression ();
				if (expression != null) {
					expressions.add (expression);
				}
			}

			return expressions;
		}
	}

}
