/* query-result.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using GLib;
using Gee;
using PeopleBackend;

namespace People {
	/**
	 * This class represents the result from an {@link AddressBook} query.
	 * <p>
	 * It consists into a temporary container filling with {@link Contact}
	 * corresponding to the query results. Chunks of those Contact objects can
	 * be issued from it.
	 */
	public class QueryResult : Object {
		public weak AddressBook address_book {
			private get;
			private construct set;
		}

		public MultiAsyncSearch<PeopleBackend.Contact> search {
			private get;
			private construct set;
		}

		public uint pending_results {
			get {return hits.size;}
		}

		private Gee.Set<string> contact_ids
				= new Gee.HashSet<string> (str_hash, str_equal);

		private Gee.Set<Contact> hits = new Gee.HashSet<Contact> ();

		private Gee.Set<MultiAsyncOperation<PeopleBackend.Contact> > contact_builders =
				new Gee.HashSet<PeopleBackend.Contact> ();

		construct {
			search.hit_found += hit_found_callback;
			search.done += done_callback;

			search.start();
		}

		private bool is_opened = false;
		private bool is_ready = false;
		private bool is_finished = false;

		public QueryResult (AddressBook address_book,
				MultiAsyncSearch<PeopleBackend.Contact> search) {
			this.address_book = address_book;
			this.search = search;
		}

		/* Public API */
		/**
		 * Signal triggered when {@link Contact} are ready to be issued.
		 *
		 * @param contact_count count of Contact ready to be issued
		 */
		public signal void ready (uint contact_count);

		/**
		 * Signal triggered when all the {@link Contact} corresponding to this
		 * QueryResult have been added to it.
		 */
		public signal void finished ();

		/**
		 * Opens this QueryResult.
		 * <p>
		 * Once this method is called, signals for this QueryResult may be
		 * triggered.
		 */
		public void open () {
			if (is_opened) {
				return;
			}

			is_opened = true;

			if (is_ready) {
				ready (hits.size);
			}

			if (is_finished) {
				finished ();
			}
		}

		/**
		 * Issues a certain count of {@link Contact} from this QueryResult.
		 * <p>
		 * If the count of currently available Contact is less than
		 * <code>contact_count</count>, then all the available Contact are
		 * issued.
		 *
		 * @param contact_count count of Contact to be issued
		 * @return              a {@link Set} of Contact
		 */
		 // TODO: complain if not opened
		public Gee.Set<People.Contact> issue (uint contact_count) {
			var chunk = new Gee.HashSet<Contact> ();

			foreach (Contact hit in hits) {
				chunk.add (hit);
				if (contact_count-- == 0) {
					break;
				}
			}

			foreach (Contact hit in chunk) {
				hits.remove (hit);
			}

			return chunk;
		}

		/**
		 * Closes this QueryResult.
		 * <p>
		 * Corresponding underlying search process is interrupted if possible.
		 * Currently available {@link Contact} are dropped. A QueryResult should
		 * not be used after this method is called.
		 */
		 // TODO: complain if not opened
		public void close () {
			search.stop ();
			hits.clear ();
 		}
		/* End of public API */

		private void done_callback (MultiAsyncSearch<PeopleBackend.Contact> search) {
			is_finished = true;

			if (is_opened) {
				finished ();
			}
		}

		private void hit_found_callback (MultiAsyncSearch<PeopleBackend.Contact> search,
				PeopleBackend.ContactSource source, PeopleBackend.Contact hit) {
			var source_id = address_book.sources.get_x (source);
			var full_contact_id = source_id + ":" + hit.id;

			// Contact already returned as part of a previous result
			if (contact_ids.contains (full_contact_id))
				return;

			// Find all the possible ids for the contact
			var all_contact_ids = new Gee.ArrayList<string> ();
			foreach (Gee.List<string> merged_contact in address_book.merged_contacts) {
				// FIXME: workaround a libgee ArrayList.contains bug
				bool contains = false;
				foreach (weak string id in merged_contact) {
					if (id == full_contact_id) {
						contains = true;
						break;
					}
				}

				if (contains) {
					foreach (weak string contact_id in merged_contact) {
						all_contact_ids.add (contact_id);
					}
					break;
				}
			}
			if (((Collection) all_contact_ids).size == 0)
				all_contact_ids.add (full_contact_id);

			// Add the ids to the known contacts set
			foreach (weak string id in all_contact_ids)
				contact_ids.add (id);

			// Build the contact
			build_contact (all_contact_ids, source_id, hit);
		}

		private void build_contact (Gee.List<string> contact_ids,
				string source_id, PeopleBackend.Contact hit) {
			var ops = new ArrayList<AsyncOperation<PeopleBackend.Contact> > ();
			string merged_contact_id = "";
			int i = 0;
			foreach (weak string contact_id in contact_ids) {
				i++;
				merged_contact_id += contact_id;
				if (i < contact_ids.size)
					merged_contact_id += "+";

				var ids = contact_id.split (":");
				AsyncOperation<PeopleBackend.Contact> op;
				if ((ids[0] == hit.id) && (ids[1] == source_id)) {
					// no need to fetch hit from its source again
					op = new AsyncOperation<PeopleBackend.Contact> ();
					op.completed_with_result (hit);
				} else {
					op = address_book.sources.get_y (ids[0]).request_contact (ids[1]);
				}
				ops.add (op);
			}

			var contact_builder = new MultiAsyncOperation<PeopleBackend.Contact> (ops);
			contact_builder.add_callback (
					contact_builder_callback,
					merged_contact_id);

			if (contact_ids.size > 1)
				debug ("Creating merged contact: %s", merged_contact_id);
			contact_builders.add (contact_builder);
		}

		private void contact_builder_callback (MultiAsyncOperation<PeopleBackend.Contact> builder,
				void* id) {
			try {
				var backend_contacts = builder.get_result ();
				hits.add (new People.Contact ((string) id, backend_contacts));

				if (is_opened) {
					ready (hits.size);
				} else {
					is_ready = true;
				}
			} catch (Error e) {
				warning ("contact_builder_callback: %s", e.message);
			}
			contact_builders.remove (builder);
		}
	}

}
