using GLib;
using Gee;
using People;
using PeopleBackend;

namespace PeopleTest {
	class AddressBook : TestIt.Suite {
		private PeopleBackend.ContactSource keyfile_source;
		private PeopleBackend.ContactSource sqlite_source;
		private bool ready_callback_called;
		private bool finished_callback_called;

		private People.AddressBook address_book;

		protected override void initialize () {
			name = "People::AddressBook";
			add_test_case ("query", test_query);
		}

		protected override void set_up () {
			// keyfile source set up
			var keyfile_factory = new PeopleBackendKeyfile.ContactSourceFactory ();
			var params = new Gee.HashMap<string, string> (str_hash, str_equal, str_equal);
			params.set ("file-path", "/tmp/people-test-address-book-keyfile.ini");
			try {
				keyfile_source = ((ContactSourceFactory) keyfile_factory).request_contact_source (params);
			} catch (ContactSourceFactoryError e) {
				stdout.printf("%s\n", e.message);
			}

			var contacts_first = new string[] {"Ali", "Johann", "Monchieu"};
			var contacts_last = new string[] {"Sabil", "Prieur", "Baboul"};
			for(int i = 0; i < contacts_first.length; i++) {
				try {
					var op = keyfile_source.request_contact (null);
					var contact = op.get_result();
					var fields = new Gee.HashMap<string, string>(str_hash, str_equal, str_equal);
					fields.set ("identity.name.first", contacts_first[i]);
					fields.set ("identity.name.last", contacts_last[i]);
					contact.set_fields (fields);
				} catch (Error e) {
					stdout.printf("%s\n", i);
				}
			}

			// sqlite source set up
			var sqlite_factory = new PeopleBackendSqlite.ContactSourceFactory ();
			params = new Gee.HashMap<string, string> (str_hash, str_equal, str_equal);
			params.set ("db-path", "/tmp/people-test-address-book-sqlite.db");
			sqlite_source = sqlite_factory.request_contact_source (params);

			contacts_first = new string[] {"Ali", "Bu", "John"};
			contacts_last = new string[] {"Sabil", "Bronze", "Doe"};
			for(int i = 0; i < contacts_first.length; i++) {
				try {
					var op = sqlite_source.request_contact (null);
					var contact = op.get_result();
					var fields = new Gee.HashMap<string, string>(str_hash, str_equal, str_equal);
					fields.set ("identity.name.first", contacts_first[i]);
					fields.set ("identity.name.last", contacts_last[i]);
					contact.set_fields (fields);
				} catch (Error e) {
					stdout.printf("%s\n", i);
				}
			}

			// sources
			var map = new Gee.HashMap<string, PeopleBackend.ContactSource> (str_hash, str_equal);
			map.set ("local-keyfile", keyfile_source);
			map.set ("local-db", sqlite_source);
			var sources = new People.BiMap<string, PeopleBackend.ContactSource> (map, str_hash, str_equal);

			// contact mapping
			var merged_contacts = new Gee.HashSet<Gee.List<string> > ();
			var merge1 = new Gee.ArrayList<string> ();
			merge1.add ("local-keyfile:Ali-Sabil");
			merge1.add ("local-keyfile:Monchieu-Baboul");
			merge1.add ("local-db:1");
			merged_contacts.add (merge1);
			var merge2 = new Gee.ArrayList<string> ();
			merge2.add ("local-keyfile:Johann-Prieur");
			merge2.add ("local-db:2");
			merged_contacts.add (merge2);

			// AddressBook construction
			address_book = new People.AddressBook (sources, merged_contacts);

			ready_callback_called = false;
			finished_callback_called = false;
		}

		protected override void tear_down () {
			keyfile_source = null;
			sqlite_source = null;

			address_book = null;
			FileUtils.unlink ("/tmp/people-test-address-book-keyfile.ini");
			FileUtils.unlink ("/tmp/people-test-address-book-sqlite.db");
		}

		private void test_query () {
			QueryResult result = address_book.query ();
			result.ready += ready_callback;
			result.finished += finished_callback;

			result.open ();
			assert_true (result.pending_results == 3);
			var result_set = result.issue (6);
			assert_true (result.pending_results == 0);
			assert_true (result_set.size == 3);
			assert_true (ready_callback_called);
			assert_true (finished_callback_called);
			result.close ();
		}

		private void ready_callback (QueryResult result, uint contact_count) {
			ready_callback_called = true;
		}

		private void finished_callback (QueryResult result) {
			finished_callback_called = true;
		}

		static int main(string[] args) {
			var suite = new AddressBook();
			return suite.run();
		}
	}
}
