using GLib;
using Gee;
using People;
using PeopleBackend;

namespace PeopleTest {
	class Contact : TestIt.Suite {
		private PeopleBackend.ContactSource keyfile_source;
		private PeopleBackend.ContactSource sqlite_source;
		private People.AddressBook address_book;

		private PeopleBackend.AsyncOperation op;

		protected override void initialize () {
			name = "People::Contact";
			add_test_case ("defined fields", test_defined_fields);
			add_test_case ("get fields", test_get_fields);
		}

		protected override void set_up () {
			// keyfile source set up
			var keyfile_factory = new PeopleBackendKeyfile.ContactSourceFactory ();
			var params = new Gee.HashMap<string, string> (str_hash, str_equal, str_equal);
			params.set ("file-path", "/tmp/people-test-contact-keyfile.ini");
			try {
				keyfile_source = ((ContactSourceFactory) keyfile_factory).request_contact_source (params);
			} catch (ContactSourceFactoryError e) {
				stdout.printf("%s\n", e.message);
			}

			var contacts_first = new string[] {"Ali", "Johann", "Monchieu"};
			var contacts_last = new string[] {"Sabil", "Prieur", "Baboul"};
			for(int i = 0; i < contacts_first.length; i++) {
				try {
					var op = keyfile_source.request_contact (null);
					var contact = op.get_result();
					var fields = new Gee.HashMap<string, string>(str_hash, str_equal, str_equal);
					fields.set ("identity.name.first", contacts_first[i]);
					fields.set ("identity.name.last", contacts_last[i]);
					contact.set_fields (fields);
				} catch (Error e) {
					stdout.printf("%s\n", i);
				}
			}

			// sqlite source set up
			var sqlite_factory = new PeopleBackendSqlite.ContactSourceFactory ();
			params = new Gee.HashMap<string, string> (str_hash, str_equal, str_equal);
			params.set ("db-path", "/tmp/people-test-contact-sqlite.db");
			sqlite_source = sqlite_factory.request_contact_source (params);

			contacts_first = new string[] { "Ali", "Bu", "John" };
			contacts_last = new string[] { "Sabil", "Bronze", "Doe" };
			var contacts_email = new string[] { "ali.sabil@NOSPAM.com",
					"spam@NOBU.fi", "johny@stars.us" };
			for(int i = 0; i < contacts_first.length; i++) {
				try {
					var op = sqlite_source.request_contact (null);
					var contact = op.get_result();
					var fields = new Gee.HashMap<string, string>(str_hash, str_equal, str_equal);
					fields.set ("identity.name.first", contacts_first[i]);
					fields.set ("identity.name.last", contacts_last[i]);
					fields.set ("communication.email.home", contacts_email[i]);
					contact.set_fields (fields);
				} catch (Error e) {
					stdout.printf("%s\n", i);
				}
			}

			// sources
			var map = new Gee.HashMap<string, PeopleBackend.ContactSource> (str_hash, str_equal);
			map.set ("local-keyfile", keyfile_source);
			map.set ("local-db", sqlite_source);
			var sources = new People.BiMap<string, PeopleBackend.ContactSource> (map, str_hash, str_equal);

			// contact mapping
			var merged_contacts = new Gee.HashSet<Gee.List<string> > ();
			var merge1 = new Gee.ArrayList<string> ();
			merge1.add ("local-keyfile:Ali-Sabil");
			merge1.add ("local-keyfile:Monchieu-Baboul");
			merge1.add ("local-db:1");
			merged_contacts.add (merge1);
			var merge2 = new Gee.ArrayList<string> ();
			merge2.add ("local-keyfile:Johann-Prieur");
			merge2.add ("local-db:2");
			merged_contacts.add (merge2);

			// AddressBook construction
			address_book = new People.AddressBook (sources, merged_contacts);
		}

		protected override void tear_down () {
			keyfile_source = null;
			sqlite_source = null;

			address_book = null;
			FileUtils.unlink ("/tmp/people-test-contact-keyfile.ini");
			FileUtils.unlink ("/tmp/people-test-contact-sqlite.db");

			op = null;
		}

		private void test_defined_fields () {
			QueryResult result = address_book.query ();
			result.finished += defined_fields_finished_callback;

			result.open ();
		}

		private void defined_fields_finished_callback (QueryResult result) {
			var contacts = result.issue (3);

			foreach (People.Contact contact in contacts) {
				if (contact.id == "local-keyfile:Ali-Sabil+local-keyfile:Monchieu-Baboul+local-db:1") {
					op = contact.get_defined_fields ();
					op.add_callback (test_defined_fields_callback);
				}
			}

			result.close ();
		}

		private void test_defined_fields_callback(AsyncOperation<Gee.Set<string> > op) {
			var fields = new string[] { "identity.name.first",
										"identity.name.last",
										"communication.email.home" };

			var result = op.get_result ();
			foreach (string field in fields) {
				assert_true (result.contains (field));
			}
		}

		private void test_get_fields () {
			QueryResult result = address_book.query ();
			result.finished += get_fields_finished_callback;

			result.open ();
		}

		private void get_fields_finished_callback (QueryResult result) {
			var fields = new Gee.HashSet<string> (str_hash, str_equal);
			fields.add ("identity.name.first");
			fields.add ("communication.email.home");

			var contacts = result.issue (3);

			foreach (People.Contact contact in contacts) {
				if (contact.id == "local-keyfile:Ali-Sabil+local-keyfile:Monchieu-Baboul+local-db:1") {
					op = contact.get_fields (fields);
					op.add_callback (test_get_fields_callback);
				}
			}

			result.close ();
		}

		private void test_get_fields_callback (AsyncOperation<Gee.Map<string, string> > op) {
			var result = op.get_result ();

			assert_true (result.size == 2);

			string first = result.get ("identity.name.first");
			assert_true (first != null);
			assert_true ((first == "Ali") || (first == "Monchieu"));

			string second = result.get ("communication.email.home");
			assert_true (second != null);
			assert_true (second == "ali.sabil@NOSPAM.com");
		}

		static int main(string[] args) {
			var suite = new Contact();
			return suite.run();
		}
	}
}
